import numpy as np
import pandas
import pyshark
from PIL import Image

def delimiterToken(pcap):
    """
    Extracts the payloads and separates them by whitespace

    Args:
        pcap: the pcap file to pull payloads from

    Returns:
        list of tokenized payloads
    """
    i = 0
    payloads = []
    # go through every packet in a pcap and add both representations
    # of the payload to two seperate lists
    for p in pcap:
        tokened,length = tokenPayload(p)
        if tokened == None:
            continue
        tokened =  list(map(lambda hex: chr(hex), tokened))
        sentence = ''.join(tokened)
        delimitedTok = sentence.split()
        payloads.append(delimitedTok)
    print(payloads[0])
    #will return payload list
    return payloads

#takes a pcap of one session and then makes a gray scale image from
#the first 90 packets or till it is 784 bytes long.
def matrix2d(pcap):
    """
    Makes a gray scale image from the first 784 bytes.
    """
    #count = 0
    payload = []
    matpayloads = []
    # go through every packet in a pcap and add both representations
    # of the payload to two seperate lists
    for p in pcap:
        tokened,length = tokenPayload(p)
        #matrixed = payMatrix(p)
        if tokened == None:
            continue
        #count += 1
        #if len(payload) > 784 or count == 90:
            #break
        matrixed = payMatrix(tokened, len(tokened))
        matpayloads.append(matrixed)
    #will return payload list as a padded matrix
    return matpayloads

#below is the corresponding function to the 224x224 papers method
'''def matrix2d(pcap):
    i = 0
    matpayloads = []
    maxpayload = 0
    # go through every packet in a pcap and add both representations
    # of the payload to two seperate lists
    for p in pcap:
        tokened,length = tokenPayload(p)
        #matrixed = payMatrix(p)
        if tokened == None:
            continue
        matrixed = payMatrix(tokened, length)
        matpayloads.append(matrixed)
    #will return payload list as a padded matrix
    return matpayloads'''

def matrix3d(pcap):
    """
    Right now just make a 3d matrix from the payload shape still needs to be decided
    """
    payloads3d = []
    # go through every packet in a pcap and add both representations
    # of the payload to two seperate lists
    for p in pcap:
        tokened,length = tokenPayload(p)
        #matrixed = payMatrix(p)
        if tokened == None:
            continue
        matrixed = payMatrix3d(tokened, length)
        payloads3d.append(matrixed)
    #will return payload list as a padded matrix
    return payloads3d


def tokenPayload(packet):
    """
        first extracts the payloads and then returns a list of the bytes of the payload as ints
    """
    payload = None
    #check if tcp and has payload
    if hasattr(packet, 'tcp'):
        prot = 'tcp'
        if hasattr(packet.tcp, "payload"):
            payload = packet.tcp.payload
    #check if udp and has payload
    elif hasattr(packet, 'udp'):
        prot = 'udp'
        if hasattr(packet.udp, "payload"):
            payload = packet.udp.payload
    if not payload:
        return None,0
    #split the payload into the individual numbers
    payload = payload.split(':')
    #convert them from hexi to integers based 10
    payload = list(map(lambda num: int(num,16), payload))
    return payload, len(payload)

def payMatrix(payArray, size):
    """
    This makes a 2-d representation of the payload makes it into a 28x28 image

    Any payload larger than this will be trimmed.
    Any smaller will be zero padded.

    Args:
        payArray: the payload as an array of ints
        size: total size of payload array

    Returns:
        2d matrix of the payload
    """
    #take several packet data split and make grayscale
    #making 28x28 so only want 784 bytes
    if size < 784:
        pads = 784 - size
        pad = [0 for i in range(pads)]
        #adding 0's to the end so we can reshape it
        payArray = payArray + pad
    elif size > 784:
        #trimming if more bytes than needed
        payArray = payArray[:784]
    #numpy reshapes for you so convert
    matrix = np.array(payArray).reshape(28,28).astype(np.uint8)
    #image = Image.fromarray(matrix)
    #return the image for the sessino
    return matrix

#below is the code to make a grayscale image for the payload
#seen in a different paper making it a size of 224x224
'''def payMatrix(payArray,size):
    #take the token version and reshape it
    #payArray,length = tokenPayload(packet)
    if not payArray:
        return None
    #rows on wireshark are len 16 so doing the same
    #want all rows to be same length
    #pads = 16 - (size%16) + (size - len(payArray))
    #like the packet vision paper where size is nx8
    pads = 8 - (size%8) + (size - len(payArray))
    #pads = 256*256 - len(payArray)
    pad = [0 for i in range(pads)]
    #adding 0's to the end so we can reshape to rows of 16
    payArray = payArray + pad
    #numpy reshapes for you so convert
    #matrix = np.array(payArray).reshape(-1,16)
    matrix = np.array(payArray).reshape(-1,8).astype(np.uint8)
    image = Image.fromarray(matrix)
    image = image.resize((224,224))
    #matrix = np.array(payArray).reshape(256,256)
    return image'''

def payMatrix3d(payArray,size):
    """
    This makes a 3-d representation of the payload making it nx8x3
    Any payload larger than this will be trimmed.
    Any smaller will be zero padded.

    Args:
        payArray: the payload as an array of ints
        size: total size of payload array

    Returns:
        3d matrix of the payload
    """

    pads = 24 - (size%24) + (size - len(payArray))
    pad = [0 for i in range(pads)]
    #adding 0's to the end so we can reshape to rows of 16
    payArray = payArray + pad
    #numpy reshapes for you so convert
    #matrix = np.array(payArray).reshape(-1,16)
    matrix = np.array(payArray).reshape(-1,8,3)
    return matrix


'''pcap = pyshark.FileCapture('../datasets/raw/combined_pcaps/bittorrent.pcap')
pcaps = matrix2d(pcap)
for i in range(10):
    Image.fromarray(pcaps[i]).save('28x28_'+str(i)+'.jpeg')'''



import argparse
import sys
import os
import pylibmagic
import magic
import pickle
import numpy as np
from pick import pick
from pandas import read_csv, concat, DataFrame
from sklearn.preprocessing import OneHotEncoder
from statistics import mode
import models.alpine as ALPINE
import models.palm as PALM
import models.maple as MAPLE
import models.date as DATE

import rexactor
import tapcap

root_cache = "cache"
alpine_cache = root_cache + "/alpine"
palm_cache = root_cache + "/palm"
maple_cache = root_cache + "/maple"
date_cache = root_cache + "/date"
results_cache = root_cache + "/results"
alpine_bin = alpine_cache + "/alpine.bin"
alpine_labels = alpine_cache + "/labels.txt"
palm_bin = palm_cache + "/palm.bin"
palm_labels = palm_cache + "/labels.txt"
maple_h5 = maple_cache + "/maple.h5"
maple_json = maple_cache + "/maple.json"
maple_labels = maple_cache + "/labels.txt"
date_h5 = date_cache + "/date.h5"
date_json = date_cache + "/date.json"
date_labels = date_cache + "/labels.txt"

def vote(voters):
    joined = np.array([np.array(v) for v in voters[0]])
    for i in range (1, len(voters)-1):
        joined = np.column_stack((joined, np.array([np.array(v) for v in voters[i]])))
    ballot = []
    for votes in joined:
        ballot.append(mode(votes.tolist()))
    return ballot

def main():
    if not os.path.exists(root_cache):
        os.makedirs(root_cache)
    if not os.path.exists(results_cache):
        os.makedirs(results_cache)

    print("Forager: A Network Training Classification Toolkit")
    title = 'Forager: A Network Training Classification Toolkit.\n\t Please choose a task: '
    options = ['tabularize packet data (TaPCAP)',
               'generate regular expression signatures (RExACtor)',
               'configure and train models (ALPINE, PALM, MAPLE, DATE)',
               'classify packets (ALPINE, PALM, MAPLE, DATE)',
               'clear current cache']

    option, index = pick(options, title, indicator='=>', default_index=0)

    if index == 0:
        print("TaPCAP - features will be extracted to tabular format.")
        another = True
        while another:
            filepath = input("PCAP file input path? ")
            while not os.path.isfile(filepath):
                print("File not found.")
                filepath = input("PCAP file input path? ")
            outpath = input("CSV file output path? ")
            while os.path.isfile(outpath):
                print("CSV file already exists, provide another path.")
                outpath = input("CSV file output path? ")
            tapcap.pcap2csv(filepath, outpath)
            ans = input("Process another file (y/n)? ")
            if ans == "y" or ans == "yes" or ans == "YES":
                another = True
            else:
                another = False

    if index == 1:
        print("RExACtor - a regular expression signature will be generated.")
        preThres = float(input("Prefix frequency threshold (0.0 - 1.0)? "))
        sufThres = float(input("Suffix frequency threshold (0.0 - 1.0)? "))
        filepath = input("File input path (PCAP, PCAPNG, or CSV)? ")
        while not os.path.isfile(filepath):
            print("File not found.")
            filepath = input("File input path (PCAP, PCAPNG, or CSV)? ")

        CSVextension = filepath[len(filepath) - 3:].lower()
        PCAPextension = filepath[len(filepath) - 4:].lower()
        PCAPNGextension = filepath[len(filepath) - 6:].lower()

        if PCAPextension == "pcap" or PCAPNGextension == "pcapng":
            tapcap.pcap2csv(filepath, "local.csv")
            filepath = "local.csv"
        elif CSVextension != "csv":
            print("Invalid source file provided. Must be .csv, .pcap, or .pcapng.")
            sys.exit()
        rexactor.main(filepath, preThres, sufThres)

    if index == 2:
        print("Entering training mode...")
        cont = input("WARNING: editing a model's configuration will override its current cache and settings. Continue (y/n)? ")
        if cont == "n" or cont == "N":
            sys.exit()

        title = 'Forager: A Network Training Classification Toolkit.\nPlease choose one or more models to train (press SPACE to mark, ENTER to continue): '
        options = ['ALPINE',
                   'PALM',
                   'MAPLE',
                   'DATE']
        selected = pick(options, title, indicator='=>', multiselect=True, min_selection_count=1)
        indexes = [i[1] for i in selected]

        if len(indexes) == 0:
            print("configuration failed - user must select at least one model.")
            sys.exit()

        another = True
        colnames=["frame_number", "time", "highest_protocol", "l4_protocol", "text", "src_ip", "src_port", "dst_ip", "dst_port", "len", "ipflags", "tos", "bytes"]
        all_colnames = ["label"] + colnames
        all_dfs = DataFrame(columns=all_colnames)
        labels = []

        while another:
            filepath = input("CSV file input path? ")
            while not os.path.isfile(filepath):
                print("File not found.")
                filepath = input("CSV file input path? ")

            label = input("Label? ")
            while label == "":
                print("No label provided.")
                label = input("Label? ")
            if label not in labels:
                labels.append(label)

            df = read_csv(filepath, delimiter="|", names=colnames, header=None)
            df['label'] = label
            all_dfs = concat([all_dfs, df])

            another_text = input("Add another file (y/n)? ")
            if another_text == "n" or another_text == "N" or another_text == "":
                another = False

        enc = OneHotEncoder()
        enc.fit(np.array(all_dfs["label"]).reshape(-1,1))
        train_labels = enc.transform(np.array(all_dfs["label"]).reshape(-1,1)).toarray()
        labels = sorted(labels)

        alpineCount = [0]
        palmCount = [0]
        mapleCount = [0]
        dateCount = [0]

        #todo: pull in the models
        if 0 in indexes:
            alpine = ALPINE.Alpine(columns=["src_ip", "src_port", "dst_ip", "dst_port", "len", "ipflags", "tos"])
            for label in labels:
                print("Adding data for label: " + label)
                curr = all_dfs.loc[all_dfs['label'] == label]
                if (curr.empty):
                    print("No data provided for label: " + label + ". Skipping this one.")
                alpine.add_bucket(curr, label, alpineCount)
                print("Generated " + str(alpineCount) + " total signatures.")
            alpine.fit()

            if not os.path.exists(alpine_cache):
                os.makedirs(alpine_cache)
            binary_file = open(alpine_bin, mode='wb+')
            pickle.dump(alpine, binary_file)
            binary_file.close()
            label_file = open(alpine_labels,'w+')
            newline = ""
            for label in labels:
                label_file.write(newline + label)
                newline = "\n"
            label_file.close()

        if 1 in indexes:
            print("training PALM...")
            palm = PALM.Palm(col_name="text")
            for label in labels:
                print("Adding data for label: " + label)
                curr = all_dfs.loc[all_dfs['label'] == label]
                palm.add_bucket(curr, label, palmCount)
                print("Generated " + str(palmCount) + " total signatures.")
            palm.fit()

            if not os.path.exists(palm_cache):
                os.makedirs(palm_cache)
            binary_file = open(palm_bin, mode='wb+')
            pickle.dump(palm, binary_file)
            label_file = open(palm_labels,'w+')
            newline = ""
            for label in labels:
                label_file.write(newline + label)
                newline = "\n"
            label_file.close()

        if 2 in indexes:
            print("training MAPLE...")
            if not os.path.exists(maple_cache):
                os.makedirs(maple_cache)
            maple = MAPLE.Maple(labels, weights_path=maple_h5, json_path=maple_json, col_name="bytes", epochs=10)
            maple.fit(all_dfs, train_labels)
            label_file = open(maple_labels,'w+')
            newline = ""
            for label in labels:
                label_file.write(newline + label)
                newline = "\n"
            label_file.close()

        if 3 in indexes:
            print("training DATE...")
            if not os.path.exists(date_cache):
                os.makedirs(date_cache)
            date = DATE.Date(labels, weights_path=date_h5, json_path=date_json, col_name="bytes", epochs=10)
            date.fit(all_dfs, train_labels)
            label_file = open(date_labels,'w+')
            newline = ""
            for label in labels:
                label_file.write(newline + label)
                newline = "\n"
            label_file.close()
        print("Training complete. Models saved to " + root_cache + ".")

    if index == 3:
        print("Entering testing mode...")
        title = 'Forager: A Network Training Classification Toolkit.\nPlease choose one or more models to test with (press SPACE to mark, ENTER to continue): '
        options = ['ALPINE',
                   'PALM',
                   'MAPLE',
                   'DATE']
        selected = pick(options, title, indicator='=>', multiselect=True, min_selection_count=1)
        indexes = [i[1] for i in selected]

        if len(indexes) == 0:
            print("configuration failed - user must select at least one model.")
            sys.exit()

        colnames=["frame_number", "time", "highest_protocol", "l4_protocol", "text", "src_ip", "src_port", "dst_ip", "dst_port", "len", "ipflags", "tos", "bytes"]
        all_colnames = ["label"] + colnames
        all_dfs = DataFrame(columns=all_colnames)
        filepath = ""

        filepath = input("CSV file input path? ")
        while not os.path.isfile(filepath):
            print("File not found.")
            filepath = input("CSV file input path? ")

        path_pieces = filepath.split("/")
        outfile = results_cache + "/" + (path_pieces[len(path_pieces)-1])[:-4] + "_results.txt"
        df = read_csv(filepath, delimiter="|", names=colnames, header=None)
        df['label'] = ""
        all_dfs = concat([all_dfs, df])

        votes = []

        if 0 in indexes:
            print("testing ALPINE")
            if not os.path.exists(alpine_bin):
                print("ERROR: ALPINE has no existing model. Please provide training data first.")
            else:
                with open(alpine_bin, mode='rb') as file: # b is important -> binary
                    alpine = pickle.loads(file.read())
                    alpine_results = alpine.predict(all_dfs)
                    votes.append(alpine_results)

        if 1 in indexes:
            print("testing PALM")
            if not os.path.exists(palm_bin):
                print("ERROR: PALM has no existing model. Please provide training data first.")
            else:
                with open(palm_bin, mode='rb') as file: # b is important -> binary
                    palm = pickle.loads(file.read())
                    palm_results = palm.predict(all_dfs)
                    votes.append(palm_results)

        if 2 in indexes:
            print("testing MAPLE")
            if not os.path.exists(maple_h5) or not os.path.exists(maple_json) or not os.path.exists(maple_labels):
                print("ERROR: MAPLE has no existing model. Please provide training data first.")
            else:
                label_file = open(maple_labels, "r")
                labels = label_file.read().split("\n")
                maple = MAPLE.Maple(labels, col_name="bytes", weights_path=maple_h5, json_path=maple_json, num_votes=10, use_existing=True)
                maple_results = maple.predict(all_dfs)
                votes.append(maple_results)

        if 3 in indexes:
            print("testing DATE")
            if not os.path.exists(date_h5) or not os.path.exists(date_json) or not os.path.exists(date_labels):
                print("ERROR: DATE has no existing model. Please provide training data first.")
            else:
                label_file = open(maple_labels, "r")
                labels = label_file.read().split("\n")
                date = DATE.Date(labels, col_name="bytes", weights_path=date_h5, json_path=date_json, num_votes=10, use_existing=True)
                date_results = date.predict(all_dfs)
                votes.append(date_results)

        results_file = open(outfile, "w+")
        newline = ""
        for v in vote(votes):
            results_file.write(newline + v)
            newline = "\n"
        print("Results available at " + str(outfile) + ".")

    if index == 4:
        answer = input("All cache files will be deleted. Proceed (y/n)? ")
        if answer == 'y' or answer == 'Y':
            os.remove(alpine_cache + "/*")
            os.remove(palm_cache + "/*")
            os.remove(maple_cache + "/*")
            os.remove(date_cache + "/*")
            print("Files deleted.")

if __name__ == "__main__":
    main()



import argparse
import os
import glob
import gzip
import zlib
import bz2
import pyshark
import nest_asyncio
import pandas as pd
nest_asyncio.apply()
from csv import writer

#----------------------#
#   Helper functions
#----------------------#

def fileToList(path):
    """
    Convert a file of strings to a list of strings

    Args:
        path: the path of the file
        encoding: the encoding of the file

    Returns:
        the list of strings
    """
    lst = []
    try:
        f = open(path, "r", encoding="latin-1")
    except:
        return lst
    for line in f:
        lst.append(line.rstrip())
    f.close()
    return lst

# write a line to the file at path
def writeToFile(path, line):
    """
    Write a line to the file at the provided path

    Args:
        path: the path for the file
        line: the line to write to file
    """
    f = open(path, 'a+')
    f.write(line + '\n')

def hexToChar(hex_string):
    """
    Convert a hex string to ASCII characters

    Args:
        hex_string: the hex string to convert

    Returns:
        ASCII string
    """
    hex_split = hex_string.split(':')
    hex_as_chars = map(lambda hex: chr(int(hex, 16)), hex_split)
    human_readable = ''.join(hex_as_chars)
    return human_readable

def parsePacketsAndCompress(input_path, label):
    """
    Parse input PCAPs and apply a random compression algorithm to their payload

    sip: Source IP Address
    dip: Destination IP Address
    sport: Source port
    dport: Destination port
    prot: Transport protocol
    dsfield: differentiated services field
    ipflags: IP flag value
    length: packet length in bytes
    label: assigned label (parameter)
    comptype: The type of compression done on the payload
    payload: packet payload, if present, compressed

    Args:
        input_path: the input path for the pcap file
        label: the label to assign to the data sample(s)

    Returns:
        pandas dataframe containing header features, payload, and label for each sample
    """
    df = pd.DataFrame(columns=['sip', 'dip', 'sport', 'dport', 'prot', 'dsfield', 'ip_flags', 'length', 'label', 'comptype', 'payload'])
    properties = []
    count = 0

    try:
        packets = pyshark.FileCapture(input_path, use_json=True)
        print("Loading PCAP input file.")

        for packet in packets:
            count += 1
            sip = None
            dip = None
            prot = None
            d_proto = None
            dsfield = None
            ip_flags = None
            payload = None
            comptype = None

            if hasattr(packet, 'ip'):
                sip = packet.ip.src
                dip = packet.ip.dst
                dsfield = packet.ip.dsfield
                ip_flags = packet.ip.flags

            if hasattr(packet, 'ipv6'):
                sip = packet.ipv6.src
                dip = packet.ipv6.dst

            if hasattr(packet, 'tcp'):
                prot = 'tcp'
                if hasattr(packet.tcp, "payload"):
                    if counter % 3 == 0:
                        payload = gzip.compress(bytes(packet.tcp.payload, 'utf-8'))
                        comptype = "gzip"
                    elif counter % 5 == 0:
                        payload = zlib.compress(bytes(packet.tcp.payload, 'utf-8'))
                        comptype = "zlib"
                    else:
                        payload = bz2.compress(bytes(packet.tcp.payload, 'utf-8'))
                        comptype = "bz2"
                    counter += 1

            elif hasattr(packet, 'udp'):
                prot = 'udp'
                if hasattr(packet.udp, "payload"):
                    if counter % 3 == 0:
                        payload = gzip.compress(bytes(packet.udp.payload, 'utf-8'))
                        comptype = "gzip"
                    elif counter % 5 == 0:
                        payload = zlib.compress(bytes(packet.udp.payload, 'utf-8'))
                        comptype = "zlib"
                    else:
                        payload = bz2.compress(bytes(packet.udp.payload, 'utf-8'))
                        comptype = "bz2"
                    counter += 1
            else:
                print("discarding non-TCP/UDP packet, detected: " + str(packet.highest_layer))
                continue

            sport = packet[packet.transport_layer].srcport
            dport = packet[packet.transport_layer].dstport
            length = packet.length
            properties.append([sip, dip, sport, dport, prot, dsfield, ip_flags, length, label, comptype, payload])

    except (UnicodeDecodeError):
        print("Could not load PCAP due to parsing error, skipping.")
        return count

    properties_frame = pd.DataFrame(properties, columns=['sip', 'dip', 'sport', 'dport', 'prot', 'dsfield', 'ip_flags', 'length', 'label', 'comptype', 'payload'])
    df = pd.concat([df, properties_frame])
    return df

def parsePackets(input_path, label):
    """
    Parse input PCAPs to get header features and payload

    sip: Source IP Address
    dip: Destination IP Address
    sport: Source port
    dport: Destination port
    prot: Transport protocol
    dsfield: differentiated services field
    ipflags: IP flag value
    length: packet length in bytes
    label: assigned label (parameter)
    payload: packet payload, if present

    Args:
        input_path: the input path for the pcap file
        label: the label to assign to the data sample(s)

    Returns:
        pandas dataframe containing header features, payload, and label for each sample
    """
    df = pd.DataFrame(columns=['sip', 'dip', 'sport', 'dport', 'prot', 'dsfield', 'ip_flags', 'length', 'label', 'payload'])
    properties = []
    count = 0

    try:
        packets = pyshark.FileCapture(input_path, use_json=True)
        print("Loading PCAP input file.")

        for packet in packets:
            count += 1
            sip = None
            dip = None
            prot = None
            d_proto = None
            dsfield = None
            ip_flags = None
            payload = None

            if hasattr(packet, 'ip'):
                sip = packet.ip.src
                dip = packet.ip.dst
                dsfield = packet.ip.dsfield
                ip_flags = packet.ip.flags

            if hasattr(packet, 'ipv6'):
                sip = packet.ipv6.src
                dip = packet.ipv6.dst

            if hasattr(packet, 'tcp'):
                prot = 'tcp'
                if hasattr(packet.tcp, "payload"):
                    payload = hexToChar(packet.tcp.payload)
            elif hasattr(packet, 'udp'):
                prot = 'udp'
                if hasattr(packet.udp, "payload"):
                    payload = hexToChar(packet.udp.payload)

            else:
                print("discarding non-TCP/UDP packet, detected: " + str(packet.highest_layer))
                continue

            sport = packet[packet.transport_layer].srcport
            dport = packet[packet.transport_layer].dstport

            length = packet.length

            #if sip is None or dip is None or sport is None or dport is None or prot is None:
            #    pass
            properties.append([sip, dip, sport, dport, prot, dsfield, ip_flags, length, label, payload])

    except (UnicodeDecodeError):
        print("Could not load PCAP due to parsing error, skipping.")
        return count

    properties_frame = pd.DataFrame(properties, columns=['sip', 'dip', 'sport', 'dport', 'prot', 'dsfield', 'ip_flags', 'length', 'label', 'payload'])
    df = pd.concat([df, properties_frame])
    return df, count

if __name__ == "__main__":

    parser = argparse.ArgumentParser(description="Process pcap file and integer data.")
    parser.add_argument("-dir", help="The directory containing the pcap files.")
    parser.add_argument("-label", help="label to assign to this pcap group.")
    args = parser.parse_args()
    out_dir = os.getcwd()
    os.chdir(str(args.dir))
    path = os.getcwd()
    filecount = 0
    total = 0
    oldtotal = 0

    output_prefix = out_dir + "/../datasets/processed"
    if not os.path.exists(output_prefix):
        os.makedirs(output_prefix)
    filename = (output_prefix + "/" + str(args.label))
    ext = str(filecount) + ".csv"

    for root,dirs,files in os.walk(path):
        pcap_files = glob.glob(os.path.join(root, "*.pcap"))

        for pcap in files:
            df, count = parsePackets(pcap, str(args.label))
            total += count
            print("Number of packets processed: %d" % total)
            filecount += 1
            ext = str(filecount) + ".csv"
            df.to_csv(filename + ext)



from models.forager import Forager
import time
import random
from sklearn.model_selection import train_test_split
from sklearn.metrics import accuracy_score, classification_report,confusion_matrix, ConfusionMatrixDisplay, cohen_kappa_score
from sklearn.preprocessing import OneHotEncoder
import matplotlib.pyplot as plt
import numpy as np
import os

import seaborn as sns
import statistics
from imblearn.under_sampling import RandomUnderSampler

USE_BALANCER     = True
USE_SAVED_MODELS = True
USE_ALPINE       = True
USE_PALM         = False
USE_MAPLE        = True
USE_DATE         = False

TEST_SIZE    = 0.2

def runTrainAndTest(X, y, ALPINE=False, PALM=False, MAPLE=False, DATE=False):
    """
    Testing harness function, for example purposes

    Args:
        X: the data as a pandas dataframe
        y: the labels as a pandas dataframe

    This class uses some global variables.

    USE_BALANCER     = use under-sampling to balance classes (will drop samples so classes are evenly distributed)
    USE_SAVED_MODELS = use a pre-trained model (faster, but you have to train the first time)
    USE_ALPINE       = use the alpine model (T/F)
    USE_PALM         = use the palm model (T/F)
    USE_MAPLE        = use the maple model (T/F)
    USE_DATE         = use the date model (T/F)

    TEST_SIZE    = if using the balancer, percentage of data to test with (80/20, 60/40)

    Using this function will generate classifier statistics, throughput rates, and a confusion matrix
    """
    global USE_SAVED_MODELS
    if not USE_SAVED_MODELS:
        if os.path.exists("models"):
            for file in os.listdir("models/*.h5"):
                try:
                    file_path = os.path.join("models", file)
                    if os.path.isfile(file_path) or os.path.islink(file_path):
                        os.unlink(file_path)
                    elif os.path.isdir(file_path):
                        shutil.rmtree(file_path)
                except Exception as e:
                    print('Failed to delete %s. Reason: %s' % (file_path, e))
            for file in os.listdir("models/*.json"):
                try:
                    file_path = os.path.join("models", file)
                    if os.path.isfile(file_path) or os.path.islink(file_path):
                        os.unlink(file_path)
                    elif os.path.isdir(file_path):
                        shutil.rmtree(file_path)
                except Exception as e:
                    print('Failed to delete %s. Reason: %s' % (file_path, e))

    print("Data count before sampling: " , len(y))
    global USE_BALANCER
    if (USE_BALANCER):
        rus = RandomUnderSampler(random_state=888)
        X, y = rus.fit_resample(X, y)
        print("Data count after sampling: " , len(y))
    else:
        print("(Sampling disabled.)")

    global TEST_SIZE
    X_train, X_test, y_train, y_test = train_test_split(X, y, test_size=TEST_SIZE, random_state=4, stratify=y)
    labels = y_train["label"].unique()

    global USE_ALPINE
    global USE_PALM
    global USE_DATE
    global USE_MAPLE
    forager = Forager(USE_ALPINE, USE_PALM, USE_MAPLE, USE_DATE)

    start = time.time()
    forager.fit(X_train, y_train)
    print("total training time in seconds: " + str(time.time() - start))

    total = 0
    MBs = 0
    for i, row in X_test.iterrows():
        total = total + int(row["length"])
        if total > 1000000:
            MBs += total/1000000
            total = 0
    if MBs == 0:
        MBs = total/1000000
    print("beginning testing phase...")

    classification_time = []
    start = time.time()
    y_pred = forager.predict(X_test)
    enc = OneHotEncoder()
    enc.fit(np.array(y_train["label"]).reshape(-1,1))
    y_pred = enc.inverse_transform(y_pred)
    end = time.time()
    print("ms per classification: " + str((((end-start)/len(y_test))*1000)))
    print("number of test samples: " + str(len(y_test)))
    print("Mb/s: " + str((MBs*8)/(end-start)))
    print("accuracy: " + str(accuracy_score(y_pred, y_test)))
    #print("cohen kappa score: " + str(cohen_kappa_score(y_test, y_pred)))
    print("classification report: \n"+ classification_report(y_test, y_pred))
    cm = confusion_matrix(y_test["label"].tolist(), y_pred)
    disp = ConfusionMatrixDisplay(confusion_matrix=cm, display_labels=labels)
    fig, ax = plt.subplots(figsize=(20, 15))
    disp.plot(ax=ax)
    tick_marks=np.arange(len(labels))
    plt.xticks(tick_marks,labels,rotation=45,fontsize=10)
    plt.yticks(tick_marks,labels,fontsize=10,rotation=0)
    plt.show()



from sklearn.preprocessing import StandardScaler
from sklearn.cluster import DBSCAN
from sklearn.preprocessing import LabelEncoder
from keras.wrappers.scikit_learn import KerasClassifier
from keras.utils import np_utils
from keras.models import model_from_json
from collections import Counter
from itertools import islice
from datasketch import MinHash
from multiprocessing import Pool, cpu_count

import numpy as np
import copy
import math
import os
import pandas as pd
import matplotlib.pyplot as plt
import keras
import tensorflow as tf


class Date():

    def __init__(self, labels, num_votes=1, col_name="payload", epochs=50, batch_size=100, weights_path='cache/date/date.h5', json_path='cache/date/date.json', use_existing=False):
        """
        Generates 3D Point Clouds and performs DBSCAN.
        """
        self.my_num_votes = num_votes
        self.weights_path = weights_path
        self.json_path = json_path
        self.col_name = col_name
        self.epochs = epochs
        self.batch_size = batch_size
        self.labels = labels
        self.use_existing = use_existing

        def initModel():
            """
            Builds the classifier model which takes the 5 statistical features derived from DBSCAN.

            This function will save the weights and model as HD5 and JSON, respectively.
            These files can be used to re-load the model and save training time.
            """
            model = keras.Sequential([
                keras.layers.Dense(5, activation=tf.nn.relu),  ## two layers of processing nodes and one node for the output (binary)
                keras.layers.Dense(5, activation=tf.nn.relu),
                keras.layers.Dense(len(self.labels), activation=tf.nn.softmax),  ## ????? labels used to just be 1
            ])
            return model

        if self.use_existing and (os.path.exists(self.weights_path) and os.path.exists(self.json_path)):
            json_file = open(self.json_path, 'r')
            loaded_model_json = json_file.read()
            json_file.close()
            model = model_from_json(loaded_model_json)
            model.load_weights(self.weights_path)
            self.nn = model
        else:
            model = initModel()
            model.compile(optimizer='adam', loss='categorical_crossentropy', metrics=['acc',keras.metrics.Precision(),keras.metrics.Recall()])
            self.nn = model

        # loss = binary_crossentrophy
        # not sure if this will work

    def save_weights(self, path=""):
        """
        Save the weights of the trained model

            Args:
                path: a path to save the weights to, optional
        """
        if (path == ""):
            path = self.weights_path
        self.cnn.save_weights(path)

    def delete_weights(self):
        """
        Delete the cached weights from previous training
        """
        os.remove(self.weights_path)

    def fit(self, data, label): ## train the data against the label.
    ####################################################################################################3
        """
        Train the model with provided data. If saved weights/JSON is found, will skip this step and use the saved values instead.

        Args:
            data: the training data, X_train (Pandas dataframe)
            label: the training labels, y_train (Pandas dataframe)
        """

        label = label[data[self.col_name].isna() == False]
        y = np.array(label)
        # retrieving data from file.
        pays = data[data[self.col_name].isna() == False][self.col_name]
        X = Date.cloudify(pays)
        X = np.asarray(X).astype(np.float32)

        self.nn.fit(X, y, epochs = self.epochs, batch_size = self.batch_size)  ## 20 was preferred on. TODO update this.
        self.nn.save_weights(self.weights_path)
        model_json = self.nn.to_json()
        with open(self.json_path, "w") as json_file:
            json_file.write(model_json)
        self.nn.summary()

    def cloudify(data):
        """
            Transform the provided data into a series of 3D point clouds, performs DBSCAN, and returns the 5 statistical features per sample

            clusterCount: number of clusters
            averageClusterSize: average size of db cluster
            standardDeviation: standard deviation of points from center
            noisePercent: percent of noise in the cloud, indicates uniformity
            totalSize: total size of the cloud

            Args:
                data: the input data, Pandas dataframe

            Returns:
                a Pandas data frame where each row corresponds 1:1 with sample, contains 5 stat features.
        """
        resultTable = pd.DataFrame(columns = ['clusterCount', 'averageClusterSize', 'standardDeviation', 'noisePercent', 'totalSize'])
        results = []
        with Pool(cpu_count()) as p:
            results = pd.DataFrame(p.map(Date.cloudgen, [str(entry) for entry in data]),
                columns = ['clusterCount', 'averageClusterSize', 'standardDeviation', 'noisePercent', 'totalSize'])

        resultTable = pd.concat([resultTable, results])
        return resultTable


    def cloudgen(entry):
        """
            Helper function for cloudify, makes 3D points from packet bytes, then performs DBSCAN.

            Args:
                entry: a single payload of a packet

            Returns:
                a list of the five statistical features (see cloudify)
        """

        window_size = 3
        tokens = []
        newTokens = []
        payloadCloud = np.array([[0,0,0]])
        payloadData = bytes(entry, 'utf-8')

        clusterCount = 0
        averageClusterSize = 0
        standardDeviation = 0
        noisePercent = 0
        totalSize = 0

        x = 0
        numPoints = 0
        for i in range(len(payloadData) - window_size + 1):
            tempList = [0,0,0]
            for element in (payloadData[i: i + window_size]):
                tempList[x] = (element)
                x = x + 1
                if(x == 3):
                    x = 0
                    payloadCloud = np.vstack((payloadCloud,np.array(tempList)))
                    numPoints = numPoints + 1  ## keep track of number of points in the cloud

        payloadCloud = np.delete(payloadCloud, (0), axis = 0)
        if(len(payloadCloud) == 0):
            return [clusterCount, averageClusterSize, standardDeviation, noisePercent, totalSize]

        clustering = DBSCAN(eps=7, min_samples=7)
        finalCloud = clustering.fit(payloadCloud)


        cloudLabels = finalCloud.labels_
        cloudLabelsCounter = Counter(finalCloud.labels_)
        #print((cloudLabelsCounter))
        trueLen = 0
        trueSize = 0
        totalSize = 0
        averageClusterSize = 0.0
        #uniqueClouds = set(finalCloud.labels_)

        clusterCount = len(cloudLabelsCounter)
        noiseCount = (cloudLabelsCounter[-1])
        entryCount = len(cloudLabels)

        #print(cloudLabelsCounter)
        for element in cloudLabelsCounter:
            if(element != -1):
                trueLen = trueLen + 1
                trueSize = totalSize + cloudLabelsCounter[element]
            totalSize = totalSize + cloudLabelsCounter[element]
        #print(uniqueClouds)

        if(trueLen > 0):
            averageClusterSize = trueSize / (trueLen)  ## ignores outliers
        else:
            averageClusterSize = 0.0
        if(totalSize > 0):
            noisePercent = noiseCount / totalSize     ## gives percent of cloud that is just noise
        else:
            noisePercent = 0.0

        ## standard deviation calculation below

        size = 0
        xminusmu = 0.0
        summation = 0.0
        standardDeviation = 0.0

        if(averageClusterSize > 0.0):
            for element in cloudLabelsCounter:
                size = cloudLabelsCounter[element]
                xminusmu = (abs(size - averageClusterSize))**2
                summation = summation + xminusmu
            standardDeviation = math.sqrt(summation / entryCount)
        else:
            standardDeviation = 0.0
        return [clusterCount, averageClusterSize, standardDeviation, noisePercent, totalSize]

    def set_votes(self, num):
        """
        Sets the number of votes for the DATE model, so you can weight it more or less in an ensemble

        Args:
            num: the number of votes to set
        """
        self.my_num_votes = num

    def predict(self,data):
        """
        Return a prediction for each entry based on the training model.

        Args:
            data: X_test, a Pandas dataframe of the data entry

        Return:
            the predicted result for each data entry
        """
        pays = data[self.col_name]
        X = Date.cloudify(pays)
        X = np.asarray(X).astype(np.float32)
        all_pred = self.nn.predict(np.array(X))
        all_results = []
        for pred in all_pred:
            result = np.zeros_like(pred)
            result[pred.argmax(0)] = 1
            result = list(result)
            all_results.append(result)
        index_results = np.argmax(all_results, axis=1)
        final = []
        for i in index_results:
            final.append([self.labels[i]] * self.my_num_votes)
        return final



import re
import farmhash
import xxhash
import mmh3
import hashlib
import numpy as np
import random
from collections import defaultdict
from datasketch import MinHash, MinHashLSHForest

#TODO: add payload delimited by whitespace to tokens as option.
#TODO: add point-cloud payload to tokens as option.
#TODO: any other useful features or ways to represent a payload?

class HashForest:
    """
    This is the parent class for ALPINE and PALM models.

    It is modeled with the same API as a sci-kit learn model, where data should be fit, then predictions made.
    """
    def __init__(self):
        """
        Initializes the number of MinHash permutations and number of votes for classification.
        """
        self.my_forest = MinHashLSHForest(num_perm=256)
        self.hash_type = "sha1"
        self.my_lookup_table = {}
        self.my_num_votes = 10

    def __init__(self, hash):
        """
        Initializes the number of MinHash permutations and number of votes for classification.
        """
        self.my_forest = MinHashLSHForest(num_perm=256)
        self.my_lookup_table = {}
        self.my_num_votes = 10

        if hash == "farmhash":
            self.hash_type = "farmhash"
        elif hash == "xxhash":
            self.hash_type = "xxhash"
        elif hash == "mmh3":
            self.hash_type = "mmh3"
        else:
            self.hash_type = "sha1"

    def hash(self, np_data):
        """
        generates a bulk of MinHashes using a configured number of permutations.

        Args:
            np_data    The dataframe containing the header features to hash

        Return:
            list of hash values
        """
        if self.hash_type == "sha1":
            encoder = np.vectorize(self._encode)
            np_data = encoder(np_data)
            return MinHash.bulk(np_data, num_perm=256)
        return MinHash.bulk(np_data, num_perm=256,hashfunc=self._hash_func)

    # We need to define a new hash function that outputs an integer that
    # can be encoded in 32 bits.
    def _hash_func(self, d):
        """
        allows for a different hash than sha1 if desired (not recommended)

        Args:
            d    the data to hash

        Return:
            the hashed value
        """
        if self.hash_type == "farmhash":
            return farmhash.hash32(d)
        elif self.hash_type == "xxhash":
            return xxhash.xxh32(d).intdigest()
        elif self.hash_type == "mmh3":
            return mmh3.hash(d)

    def _encode(self, x):
        """
        data must be encoded to utf-8 for hashing if it is not already

        Args:
            x    the data to encode

        Return:
            the encoded value
        """
        return x.encode('utf-8')

    def _process_data(self, data):
        """
        dummy method, children may override if additional processing needs to be done before data is hashed.

        Return:
            additionally processed data
        """
        return data

    def add_bucket(self, data, label, count):
        """
        Adds training data to the model with the label. You should add a bucket for each class when building a model.

        Args:
            data    the dataframe, one row for each packet sample
            label   the label to assign to this data sample, i.e. what its classified as, ground truth
            count   a recurring value used as the sample's index. One label has multiple indexes underneath it.
        """
        minhashes = self.hash(self._process_data(data))
        for m in minhashes:
            # add the hash with its index to the forest
            self.my_forest.add(count[0], m)
            self.my_lookup_table[count[0]] = label
            count[0] += 1

    def fit(self):
        """
        Calls the indexing function which builds the finalized forest model for searching. Data cannot be added after calling, model cannot be searched before calling.
        """
        self.my_forest.index()

    def set_votes(self, num):
        """
        Sets a different number of votes for classifying if desired

        Args:
            num    number of votes to set
        """
        self.my_num_votes = num

    def predict(self, data):
        """
        Makes the predictions calling the classifier and looking up the index in the built forest.

        Args:
            data    the y_test data to make predictions on

        Return:
            a list of lists where each list is the forests' votes for a sample from the input data
        """
        data = self._process_data(data)
        minhashes = self.hash(data)
        all_arrs = []
        first = True
        for m in minhashes:
            arr = np.array(self.my_forest.query(m, self.my_num_votes))
            lookup_results = []
            for ret in arr:
                lookup_results.append(self.my_lookup_table[ret])

            if len(lookup_results) < self.my_num_votes:
                remain = self.my_num_votes - len(lookup_results)
                #print("WARNING: no close proximity match found. Using random result")
                choice = random.choice(range(0, len(self.my_lookup_table)))
                supplement = [self.my_lookup_table[choice]] * remain
                lookup_results = lookup_results + supplement

            all_arrs.append(lookup_results)
        return all_arrs



from models.hashforest import HashForest
import numpy as np
from datasketch import MinHash

class Palm(HashForest):
    """
    Classifies packets based on a hash of the payload.
    """
    def __init__(self, hash="sha1", col_name="text"):
        """
        Initialize the PALM model.

        Args:
            hash     optional parameter to change the hash type, can be xxhash, farmhash, or mm3hash
            col_name the name of the column containing the text payload
        """
        HashForest.__init__(self, hash)
        self.col_name = col_name

    #TODO: instead of reading these from csv, should pull directly from pcap
    def hash(self, data):
        """
        Overridden function which first delimits the payload and then generates the hash from it as tokens

        Args:
            data   the input dataframe

        Return:
            list of the hash values from MinHash
        """
        tokens = []
        for i, row in data.iterrows():
            raw_tokens = str(row[self.col_name]).split()
            if (self.hash_type == "sha1"):
                encoded_tokens = []
                for token in raw_tokens:
                    encoded_tokens.append(token.encode('utf-8'))
                tokens.append(encoded_tokens)
            else:
                tokens.append(raw_tokens)
        if (self.hash_type == "sha1"):
            return MinHash.bulk(tokens, num_perm=256)
        return MinHash.bulk(tokens, num_perm=256,hashfunc=self._hash_func)



from models.hashforest import HashForest
import numpy as np

#TODO: add payload delimited by whitespace to tokens as option.
#TODO: add point-cloud payload to tokens as option.
#TODO: any other useful features or ways to represent a payload?

class Alpine(HashForest):
    """
    Classifies packets based on a hash of header features
    """
    def __init__(self, hash="sha1", columns=["sip","dip","sport","dport","prot","length","dsfield","ip_flags"]):
        """
        Initializes the ALPINE model.

        Args:
            hash    optional parameter to use a different hash function, options are xxhash, farmhash, m3hash
            columns optional parameter to list which columns from input dataframe to select for hash values
        """
        HashForest.__init__(self, hash)
        self.columns = columns
        print("Using these features for ALPINE: " + str(columns))

    def _process_data(self, data):
        """
        Extracts a subset of the data columns provided.

        Args:
            data    the dataframe to process

        Return:
            the columns extracted
        """
        values = data[self.columns]
        values = values.astype('str').fillna("")
        return values

    #TODO: Instead of reading these from csv, should pull from pcap
    def hash(self, data):
        """
        overridden function to call the hash function to make MinHashes

        Args:
            data the dataframe, pre-processed

        Return:
            The hashes as a numpy array
        """
        return np.array(HashForest.hash(self, data))



import tensorflow as tf
from tensorflow import keras
import numpy
import glob
import os
import util.payload
import sklearn
from sklearn.model_selection import train_test_split
from keras.models import model_from_json
import numpy as np

class Maple:
    def __init__(self, labels, num_votes=1, col_name="payload", epochs=50, batch_size=100, weights_path='cache/maple/maple.h5', json_path='cache/maple/maple.json', use_existing=False):
        """
        Generates 2D matrix images and classifies using CNN.
        """
        self.my_num_votes = num_votes
        self.weights_path = weights_path
        self.json_path = json_path
        self.col_name = col_name
        self.epochs = epochs
        self.batch_size = batch_size
        self.labels = labels
        self.use_existing = use_existing

        def make_model(inputSize):
            """
            Builds the classifier model.

            Args:
                inputSize: the number of features, typically the number of pixels of image.

            This function will save the weights and model as HD5 and JSON, respectively.
            These files can be used to re-load the model and save training time.
            """
            inputs = keras.Input(inputSize)
            conv1 = keras.layers.Conv2D(32,(3,3),padding = 'same', activation = 'relu')(inputs)
            add1 = tf.pad(inputs, tf.constant([[0,0],[0,0],[0,0],[0,31]]),'CONSTANT')
            res1 = keras.layers.Add()([conv1,add1])
            conv2 = keras.layers.Conv2D(32,(3,3),strides=2,padding = 'same', activation = 'relu')(res1)
            add2 = keras.layers.MaxPooling2D(2)(res1)
            res2 = keras.layers.Add()([conv2,add2])
            conv3 = keras.layers.Conv2D(64,(3,3),padding = 'same',activation = 'relu')(res2)
            flatten = keras.layers.Flatten()(conv3)
            dense1 = keras.layers.Dense(64, activation = 'relu')(flatten)
            drop = keras.layers.Dropout(.2)(dense1)
            dense2 = keras.layers.Dense(32, activation = 'relu')(drop)
            drop2 = keras.layers.Dropout(.2)(dense2)
            output = keras.layers.Dense(len(self.labels), activation = 'softmax')(drop2)
            maple = keras.Model(inputs,output)
            return maple

        if (self.use_existing and os.path.exists(self.weights_path) and os.path.exists(self.json_path)):
            json_file = open(self.json_path, 'r')
            loaded_model_json = json_file.read()
            json_file.close()
            maple = model_from_json(loaded_model_json)
            maple.load_weights(self.weights_path)
            print("Using existing model at " + str(self.json_path) + ".")
            self.cnn = maple
        else:
            maple = make_model((28,28,1))
            maple.compile(optimizer = 'adam', loss = 'categorical_crossentropy', metrics = ['acc',keras.metrics.Precision(),keras.metrics.Recall()])
            self.cnn = maple

    def save_weights(self, path=""):
        """
        Save the weights of the trained model

            Args:
                path: a path to save the weights to, optional
        """
        if (path == ""):
            path = self.weights_path
        self.cnn.save_weights(path)

    def delete_weights(self):
        """
        Delete the cached weights from previous training
        """
        os.remove(self.weights_path)

    def fit(self, data, label):
        pays = data[data[self.col_name].isna() == False][self.col_name]
        label = label[data[self.col_name].isna() == False]
        pays =  list(map(lambda pay: list(map(lambda c: int(ord(c)),list(pay))), pays))
        X = []
        if len(pays) == 0:
            pays = [[0]]
        for pay in pays:
            if len(pay) < 784:
                pads = 784 - len(pay)
                pad = [0 for i in range(pads)]
                #adding 0's to the end so we can reshape it
                mat = pay + pad
            elif len(pay) > 784:
                #trimming if more bytes than needed
                mat = pay[:784]
            mat = np.reshape(mat,(28,28))
            X.append(mat)

        self.cnn.fit(np.array(X), np.array(label), epochs = self.epochs, batch_size = self.batch_size)
        self.cnn.save_weights(self.weights_path)
        model_json = self.cnn.to_json()
        with open(self.json_path, "w") as json_file:
            json_file.write(model_json)
        self.cnn.summary()

    def predict(self, data):
        """
        Return a prediction for each entry based on the training model.

        Args:
            data: X_test, a Pandas dataframe of the data entry

        Return:
            the predicted result for each data entry
        """
        data[self.col_name] = data[self.col_name].fillna(' ')
        pays = data[self.col_name]
        #pays = data[data['payload'].isna() == False]['payload']
        #makeing payloads form string to ints
        pays = list(map(lambda pay: list(map(lambda c: int(ord(c)),list(pay))), pays))
        X = []
        if len(pays) == 0:
            pays = [[0]]
        for pay in pays:
            if len(pay) < 784:
                pads = 784 - len(pay)
                pad = [0 for i in range(pads)]
                #adding 0's to the end so we can reshape it
                mat = pay + pad
            elif len(pay) > 784:
                #trimming if more bytes than needed
                mat = pay[:784]
            mat = np.reshape(mat,(28,28))
            X.append(mat)
        all_pred = self.cnn.predict(np.array(X))
        all_results = []
        for pred in all_pred:
            result = np.zeros_like(pred)
            result[pred.argmax(0)] = 1
            result = list(result)
            all_results.append(result)
        index_results = np.argmax(all_results, axis=1)

        final = []
        for i in index_results:
            final.append([self.labels[i]] * self.my_num_votes)
        return final



import setuptools

with open("README.md", "r", encoding = "utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name = "forager-toolkit",
    version = "1.0.3",
    author = "Maya Kapoor",
    author_email = "mkapoor1@uncc.edu",
    description = "A network traffic classification toolkit",
    long_description = long_description,
    long_description_content_type = "text/markdown",
    url = "https://github.com/mayakapoor/forager",
    project_urls = {
        "Bug Tracker": "https://github.com/mayakapoor/forager/issues",
        "Documentation": "https://forager-toolkit.readthedocs.io/en/latest/"
    },
    classifiers = [
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    py_modules=[
        "forager"
    ],
    entry_points={
        "console_scripts": [
            "forager = forager:main",
        ]
    },
    package_dir = {"": "src"},
    packages = setuptools.find_packages(where="src"),
    install_requires=[
          'tapcap',
          'rexactor'
      ],
    python_requires = ">=3.6"
)
